package gameObjects {

    import flash.net.URLLoader;
    import flash.net.URLRequest;
    import flash.events.*;
    import flash.display.*;
    import mx.controls.*
    import mx.core.*
    import mx.containers.*;
    import flash.xml.*;    
    import flash.geom.Point;

    public class MapRoom extends Canvas {
        private var mapCols:uint;
        private var mapRows:uint;
        private var mapTXT:String;
        private var feedback:String = "";

        private var gameInitialized:Boolean = false;
        private var picksLoaded:Boolean = false;
        private var mapLoaded:Boolean = false;
        private var mapRead:Boolean = false;
        private var gameOverFlag:Boolean = false;
        
        private var mxmlApp:Application = Application(Application.application);
        private var tileArray:Array = new Array();
        private var pickArray:Array = new Array();
        private var pickableArray:Array = new Array();
        private var mapBox:Canvas;
        private var avatar:Avatar;

        private const PIX_FOLDER:String = "tilegame_assets/images/";
        private const LOC_XSTART:uint = 6;
        private const LOC_YSTART:uint = 5;

        private const TILE_W:uint = 70;
        private const TILE_H:uint = 70;

        // Alert Messages
        private const MSG_DELAY:String = "To move, you must CLICK THE MOUSE on the place " + "you want to go.";
        private const MSG_LONG_DELAY:String = "To move, you must CLICK THE MOUSE on the place " + "you want to go.";
        private const MSG_NOT_INITIALIZED:String = "Please wait a moment for the game to begin.";
        private const MSG_DIM_WH:String = "The parameters for the width and Height of the room must have positive values."
        private const MSG_ROOM_DEF:String = "The Room definition doesn't seem to be of the correct dimension.";
        private const MSG_CONGRATULATIONS:String = "Game Over\n\nCongratulations, you have picked all target words";


        // Shouldn't be altered
        private const GAMEOVER:uint = 0;

        public function MapRoom() {
            this.mapCols = 0
            this.mapRows = 0
        }

        public function startGame():void {
            var ready:Boolean = true;
            
            if (mapLoaded == false) { 
                ready = false;
                loadMap(); 
            } 

            if (mapLoaded && picksLoaded==false) { 
                ready = false;
                feedback += "map loaded" + "\n";
                loadPicks(); 
            } 
            
            if (mapLoaded && picksLoaded && mapRead == false){ 
                ready = false;
                feedback += "picks loaded" + "\n";
                readMap(); 
            } 
            
            if (ready) {
              gameInitialized = true;
              feedback += "map read" + String(mapLoaded) + " " + String(picksLoaded) + " " + String(mapRead) + "\n";
              // mx.controls.Alert.show(feedback)
              
              mapBox = new Canvas();
              drawTiles();
              placeInRoom();
              avatar = new Avatar()
              avatar.tileArray = tileArray
              avatar.seekLoc = new Point(LOC_XSTART, LOC_YSTART)
              avatar.startLevel();
              mapBox.addChild(avatar)

              var mapCanvas:Canvas;
              mapCanvas = new Canvas
              mapCanvas.addChild(mapBox)
              mxmlApp.addChild(mapCanvas)
              mapCanvas.setStyle("borderStyle", "solid")
              mapCanvas.setStyle("borderThickness", "3")
              
            }
        }

        private function loadMap():void {
            var mapLoader:URLLoader;
            mapLoader = new URLLoader();
            mapLoader.addEventListener(Event.COMPLETE, loadedMap);
            mapLoader.load(new URLRequest("tilegame_assets/data/map.txt"));
        }


        private function loadedMap(e:Event):void {
            var lengthB4:uint;
            var tileQty:uint;
            var pattern:RegExp;
            
            mapTXT = new String(e.target.data)
            lengthB4 = mapTXT.length
          	pattern = new RegExp("[\n\r]+", "gi");
            mapTXT = mapTXT.replace(pattern, "");
            tileQty = mapTXT.length;
            mapRows = lengthB4 - tileQty;
            mapCols = uint(tileQty /mapRows); 
            
            if (checkValues() > 1) { return; }
            
            mapLoaded = true;
			      startGame();
            
        }

        public function loadPicks():void {
          var picksLoader:URLLoader;
          picksLoader = new URLLoader();
          picksLoader.addEventListener(Event.COMPLETE, loadedPicks);
          picksLoader.load(new URLRequest("tilegame_assets/data/picks.txt"));
        }
        
        private function loadedPicks(e:Event):void {
          var i:uint
          var itemArray:Array = new Array(); 
          var valObj:Object; 
          var valArray:Array = new Array(); 
          
          itemArray = String(e.target.data).split("\n"); 

      		for (i=0; i < itemArray.length; i++) {
        		  valArray = itemArray[i].split("\t")
              valObj = new Object()
        		  valObj.tag = "" + valArray[0]
        		  valObj.cat = valArray[1]
        		  if (valObj.tag != undefined && valObj.cat != undefined) {  
        		    pickArray[i] = valObj;
        		  }
      		}
      		
      		picksLoaded = true;
      		startGame();
        }
        
        private function readMap():void {
            var tileTXT:String = ""; 
            var tileCode:String = "";
            
            var x:uint; 
            var y:uint;
            
            for (y = 0; y < mapRows; y++) {
                tileArray[y] = new Array();
            }
            for (y = 0; y < mapRows; y++) {
                for (x = 0; x < mapCols; x++) {
                    tileCode = String(mapTXT.charAt((y*mapRows) + x));
                    tileArray[x][y] = new MapTile(tileCode, new Point(x,y));
                    if (tileArray[x][y].pickable) {
                        pickableArray.push(tileArray[x][y]);
                    }
                    if (tileArray[x][y].walkable) {
                        tileArray[x][y].addEventListener(MouseEvent.CLICK, onTileClicked)
                    }
                }
            }
            mapRead = true;
            startGame()
            
        }
        public function onTileClicked(e:MouseEvent):void {
          var tileClicked:MapTile;
          var interrupt:Boolean = false;
          tileClicked = e.currentTarget as MapTile;

          if(!gameInitialized) {
            showWarning("MSG_NOT_INITIALIZED", 1)
            interrupt = true;
          }

          if (gameOverFlag) {
            // startGame();
            interrupt = true;
          }

          if (!interrupt) {
            avatar.seekLoc = new Point(tileClicked.xyLoc.x,tileClicked.xyLoc.y);
            avatar.movedFlag = true;
            avatar.startWalking();
          }
            
        }
        
        
        private function drawTiles():void {
            var x:uint; 
            var y:uint;

            for (x = 0; x < mapCols; x++) {
                for (y = 0; y < mapRows; y++) {
                    tileArray[x][y].x = (x) * TILE_W;
                    tileArray[x][y].y = (y) * TILE_H;
                    mapBox.addChild(tileArray[x][y]);
                }
            }
        }
        
        private function placeInRoom():void {
            // construct array so that can ensure random pick without replacement
            var i:uint; 
            var msg:String = "";
            var rnd:Array = new Array();
            
            // creating an array of numbers 
            for (i=0; i < pickableArray.length; i++) { rnd[i] = i }

            // suffle the array of pickables
            for (i=pickableArray.length; i > 0; ) {
                var j:int = Math.floor(Math.random() * --i);
                var x:uint = rnd[i]; rnd[i] = rnd[j]; rnd[j] = x;
            }

            // visual feedback on sort
            // for (i=0; i < pickableArray.length; i++) { msg += String(rnd[i]) + " " }
            // mx.controls.Alert.show(msg)

            for (i=0; i < pickArray.length; i++) {
               var r:uint = rnd[i]
                pickableArray[r].makePick(pickArray[i].tag, pickArray[i].cat)
            }

        }
        /* ------------------------------------------------------
        	checkValues
        	Make sure all parameters provided by the user conform to expectations
        ------------------------------------------------------ */
        private function checkValues():uint {
          // the width and height of the room are ill-defined
          if (mapCols < 1 || mapRows < 1) {
            showWarning(MSG_DIM_WH, 0);
            return 2;
          }
          // the array with the definition of the room is ill-defined
        	if (mapTXT.length != (mapCols * mapRows)) {  
        		showWarning(MSG_ROOM_DEF+"\n"+mapTXT, 0); 
        		return 3;
        	}
        	// no problem
        	return 1;
        }    

        private function showWarning(msg:String, mode:uint):void {
          if(mode == 1) {
            mx.controls.Alert.show(msg);
          } else {
            mx.controls.Alert.show(msg);
            // warningBox.text = msg;
          }
        }
        

    }
}
